#!/bin/bash
# AppRun script for GoogleChrome Rox-style AppDir
# Copyright 2013 Gilbert Ashley <amigo@ibiblio.org>
# This script is released under the GNU General Public License Version 2.
# Anyone is free to use and/or modify this program for their own use.

# If you modify it and distribute it, please leave the above Copyright
# notice intact and add your own below, like this:
# Copyright ???? Your Name <your email address>
# This is a modified version of the original program.
# Please do not bother the original author if you
# are experiencing problems with it, or have found a bug.
# List your changes here:
#

# This AppDir sets up a union mount/filesystem and then chroots into it
# to execute the program without having to install the program.
# It uses unionfs-fuse and fakechroot programs for the real work.

VERSION=0.2

READ_WRITE=$HOME/Choices/GoogleChrome/write
UNION=$HOME/Choices/GoogleChrome/union
# Both the write directory and the mount-point for the union need
# to be writable by the user and, at best,  should be unique to this app
mkdir -p $READ_WRITE
mkdir -p $UNION

# we need to know where this AppDir is to find the 'app' directory
HERE=$(dirname $0)

# Create the union. It consists of 3 layers:
# The read-write directory (write) -must be writable by the user
# The package content (app) takes precendence over the system
# The easiest approach is to make all of the root '/' available
# Some applications might still require 'mount --bind', before and/or after the union is mounted,
# in order to amke their contents properly available inside the chroot.
# Or individual directories can be bind'ed in the union, instead of layering '/'
unionfs-fuse -o nonempty -o allow_root -o cow $READ_WRITE=RW:$HERE/app=RO:/=RO $UNION
# Since the 'app' directory takes precedence over any files in '/', you can include newer or
# different versions of system libs right in the 'app' dir -or any libraries required by
# this program which are not in the main system ('/').
# if you want to include some general directory location with more libs,
# then use some thing like this:
# LIBS=/path/to/libdir
#unionfs-fuse -o nonempty -o allow_root -o cow $READ_WRITE=RW:$HERE/app=RO:$LIBS=RO:/=RO $UNION

# chroot into the union mount and execute the desired program
fakechroot chroot $UNION /usr/bin/google-chrome --no-sandbox 2> /dev/null

# a normal manual chroot might do this instead:
# fakechroot chroot union /bin/sh
# Note that we cannot use 'exec fakechroot ...' because we need to clean up
# and undo the union mount after the program is executed.
# We also cannot background the program 'fakechroot ... &' because
# we also need to wait for the program to exit before cleaning up

# undo the unionfs mount
fusermount -u -z $UNION
#fakechroot umount $UNION

# clean up zombie dbus crap,
# but be quiet about it.
# Of course this is gonna kill all your dbus connections :-)
# Hope nothing else was using it!
# probably need to try using process group info
# pkill -KILL dbus 2> /dev/null
# here it is
# pkill -KILL -g $$
pkill -KILL defunct 2> /dev/null
pkill -KILL dbus-daemon 2> /dev/null
pkill -KILL dbus-launch 2> /dev/null

# The google-chrome program leaves behind 2  bad links and 1 socket 
# The program cannot be re-started if these are not removed
rm -f $HOME/.config/google-chrome/Singleton*
# there is also a file $HOME/.config/google-chrome/chrome_shutdown_ms.txt
rm -f $HOME/.config/google-chrome/chrome_shutdown_ms.txt
